@@ -259,6 +259,8 @@ tree ffecom_constantunion (ffebldConstantUnion *cu, ffeinfoBasictype bt,
 tree ffecom_decl_field (tree context, tree prevfield, char *name,
 			tree type);
 #endif
+void ffecom_close_include (FILE *f);
+int ffecom_decode_include_option (char *spec);
 void ffecom_end_transition (void);
 void ffecom_exec_transition (void);
 void ffecom_expand_let_stmt (ffebld dest, ffebld source);
@@ -279,9 +281,11 @@ tree ffecom_list_ptr_to_expr (ffebld list);
 tree ffecom_lookup_label (ffelab label);
 tree ffecom_modify (tree newtype, tree lhs, tree rhs);
 #endif
+void ffecom_file (char *name);
 void ffecom_notify_init_storage (ffestorag st);
 void ffecom_notify_init_symbol (ffesymbol s);
 void ffecom_notify_primary_entry (ffesymbol fn);
+FILE *ffecom_open_include (char *name, ffewhereLine l, ffewhereColumn c);
 #if FFECOM_targetCURRENT == FFECOM_targetGCC
 void ffecom_pop_calltemps (void);
 void ffecom_pop_tempvar (tree var);
@@ -309,8 +313,12 @@ tree ffecom_which_entrypoint_decl (void);
    declaring them in some .h file.  */
 
 #if FFECOM_targetCURRENT == FFECOM_targetGCC
+extern int flag_pedantic_errors;
 void emit_nop (void);
 void announce_function (tree decl);
+void assemble_variable (tree decl, int top_level, int at_end,
+			int dont_output_data);
+void assemble_zeros (int size);
 int count_error (int warningp);
 void error (char *s, ...);
 void expand_decl (tree decl);
@@ -322,12 +330,15 @@ void fatal (char *s, ...);
 void init_function_start (tree subr, char *filename, int line);
 void make_function_rtl (tree decl);
 void make_decl_rtl (tree decl, char *asmspec, int top_level);
+void make_var_volatile (tree var);
 int mark_addressable (tree expr);
 void output_inline_function (tree fndecl);
 void pop_function_context (void);
+void pop_momentary_nofree (void);
 void preserve_initializer (void);
 void print_node (FILE *file, char *prefix, tree node, int indent);
 void push_function_context (void);
+void push_obstacks (struct obstack *current, struct obstack *saveable);
 void put_var_into_stack (tree decl);
 void remember_end_note (tree block);
 void rest_of_compilation (tree decl);
