@@ -816,8 +816,7 @@ ffedata_eval_integer1_ (ffebld expr)
       break;
 
     case FFEBLD_opPAREN:
-      error = ffedata_eval_integer1_ (ffebld_left (expr));
-      break;
+      return ffedata_eval_integer1_ (ffebld_left (expr));
 
 #if 0				/* ~~ no idea how to do this */
     case FFEBLD_opPERCENT_LOC:
@@ -1107,7 +1106,9 @@ ffedata_gather_ (ffestorag mst, ffestorag st)
 
   if ((((b = ffesymbol_init (s)) == NULL)
        && ((b = ffesymbol_accretion (s)) == NULL))
-      || (ffebld_op (b) == FFEBLD_opANY))
+      || (ffebld_op (b) == FFEBLD_opANY)
+      || ((ffebld_op (b) == FFEBLD_opCONVERT)
+	  && (ffebld_op (ffebld_left (b)) == FFEBLD_opANY)))
     return;			/* Nothing to do. */
 
   /* b now holds the init/accretion expr. */
