@@ -394,6 +394,8 @@ again:				/* :::::::::::::::::::: */
 	ffetargetAlign m;	/* Ignored. */
 
 	sym = ffebld_symter (expr);
+	if (ffesymbol_basictype (sym) == FFEINFO_basictypeANY)
+	  return FALSE;
 
 	if (value < 0)
 	  {			/* Really invalid, as in A(-2:5), but in case
@@ -454,6 +456,9 @@ again:				/* :::::::::::::::::::: */
 	  return FALSE;
 
 	sym = ffebld_symter (symexp);
+	if (ffesymbol_basictype (sym) == FFEINFO_basictypeANY)
+	  return FALSE;
+
 	if (ffesymbol_size (sym) == FFETARGET_charactersizeNONE)
 	  width = 1;
 	else
@@ -549,6 +554,10 @@ again:				/* :::::::::::::::::::: */
 	else
 	  sym = NULL;
 
+	if ((sym != NULL)
+	    && (ffesymbol_basictype (sym) == FFEINFO_basictypeANY))
+	  return FALSE;
+
 	if (begin == NULL)
 	  value = 0;
 	else
