@@ -35,12 +35,11 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
    2.  Make sure `-lf2c -lm' is at the end of the list.
 
-   3.  Make sure each `-lf2c' is followed by `-lm'.
-
-   4.  Make sure each `-lm' is followed by `-lf2c'.
+   3.  Make sure each time `-lf2c' or `-lm' is seen, it forms
+       part of the series `-lf2c -lm'.
 
    #1 is not done if `-xfoo' is in effect (where foo is not "none").
-   #2, #3, and #4 are not done if `-nostdlib' or any option that disables
+   #2 and #3 are not done if `-nostdlib' or any option that disables
    the linking phase is present, or if `-xfoo' is in effect.  Note that
    -v by itself disables linking.
 
@@ -409,8 +408,7 @@ main (argc, argv)
 
   /* 0 => initial/reset state
      1 => last arg was -l<library>
-     2 => last arg was -lm but arg preceding that not -l<library>
-     3 => last two args were -l<library> -lm.  */
+     2 => last two args were -l<library> -lm.  */
   int saw_library = 0;
 
   /* Initialize for append_arg().  */
@@ -545,14 +543,12 @@ main (argc, argv)
 		  || strcmp (argv[i], "-lmath") == 0)
 		{
 		  if (saw_library == 1)
-		    saw_library = 3;	/* -l<library> -lm. */
+		    saw_library = 2;	/* -l<library> -lm. */
 		  else if (library)
 		    {
-		      saw_library = 3;	/* -l<library> -lm. */
 		      append_arg (library);
+		      saw_library = 2;	/* -l<library> -lm. */
 		    }
-		  else
-		    saw_library = 2;	/* -lm. */
 		}
 	      else if ((library != NULL)
 		       && (strcmp (argv[i], library) == 0))
@@ -574,13 +570,15 @@ main (argc, argv)
     {				/* Doing a link and no -nostdlib. */
       if (saw_speclang)
 	append_arg ("-xnone");
-      if (!(saw_library & 1))
+      switch (saw_library)
 	{
+	case 0:
 	  append_arg (library);
-	  saw_library = 1;
+	case 1:
+	  append_arg ("-lm");
+	default:
+	  break;
 	}
-      if (!(saw_library & 2))
-	append_arg ("-lm");
     }
 
   append_arg (NULL);
