@@ -256,6 +256,7 @@ static struct _ffeintrin_name_ ffeintrin_names_[]
   {"IISHFT", "iishft", "IIShft", FFEINTRIN_genNONE, FFEINTRIN_specNONE,},	/* VXT */
 {"IISHFTC", "iishftc", "IIShftC", FFEINTRIN_genNONE, FFEINTRIN_specIISHFTC,},	/* VXT */
   {"IISIGN", "iisign", "IISign", FFEINTRIN_genNONE, FFEINTRIN_specIISIGN,},	/* VXT */
+  {"IMAG", "imag", "Imag", FFEINTRIN_genIMAG, FFEINTRIN_spec_IMAG_C_F2C,},	/* F2C */
   {"IMAX0", "imax0", "IMax0", FFEINTRIN_genNONE, FFEINTRIN_specIMAX0,},	/* VXT */
   {"IMAX1", "imax1", "IMax1", FFEINTRIN_genNONE, FFEINTRIN_specIMAX1,},	/* VXT */
   {"IMIN0", "imin0", "IMin0", FFEINTRIN_genNONE, FFEINTRIN_specIMIN0,},	/* VXT */
@@ -944,6 +945,7 @@ ffeintrin_check_loc_ (ffebld arglist)
   info = ffebld_info (arg1);
   if ((ffeinfo_kind (info) != FFEINFO_kindENTITY)
       || ((ffebld_op (arg1) != FFEBLD_opSYMTER)
+	  && (ffebld_op (arg1) != FFEBLD_opSUBSTR)
 	  && (ffebld_op (arg1) != FFEBLD_opARRAYREF)))
     return FFEBAD_INTRINSIC_REF;
 
