@@ -55,6 +55,7 @@ ffelexToken ffesta_construct_name;
 ffelexToken ffesta_label_token;	/* Pending label stuff. */
 bool ffesta_seen_first_exec;
 bool ffesta_is_entry_valid = FALSE;	/* TRUE only in SUBROUTINE/FUNCTION. */
+bool ffesta_line_has_semicolons = FALSE;
 
 /* Simple definitions and enumerations. */
 
@@ -116,7 +117,6 @@ static bool ffesta_confirmed_current_ = FALSE;
 static bool ffesta_confirmed_other_ = FALSE;
 static ffestaPossible_ ffesta_confirmed_possible_;
 static bool ffesta_current_shutdown_ = FALSE;
-static bool ffesta_last_token_was_semi_ = FALSE;
 #if !FFESTA_ABORT_ON_CONFIRM_
 static bool ffesta_is_two_into_statement_ = FALSE;	/* For IF, WHERE stmts. */
 static ffelexToken ffesta_twotokens_1_;	/* For IF, WHERE stmts. */
@@ -434,7 +434,7 @@ next_handler:			/* :::::::::::::::::::: */
       switch (ffelex_token_type (t))
 	{
 	case FFELEX_typeCHARACTER:
-	  ffelex_set_expecting_hollerith (0, FALSE, '\0',
+	  ffelex_set_expecting_hollerith (0, '\0',
 					  ffewhere_line_unknown (),
 					  ffewhere_column_unknown ());
 	  ffesta_current_handler_
@@ -592,7 +592,7 @@ ffesta_second_ (ffelexToken t)
 #if FFESTR_F90
     case FFESTR_firstALLOCATABLE:
       ffestb_args.dimlist.len = FFESTR_firstlALLOCATABLE;
-      ffestb_args.dimlist.bad = FFEBAD_INVALID_ALLOCATABLE_STMT;
+      ffestb_args.dimlist.badname = "ALLOCATABLE";
       ffesta_add_possible_nonexec_ ((ffelexHandler) ffestb_dimlist);
       break;
 #endif
@@ -600,7 +600,7 @@ ffesta_second_ (ffelexToken t)
 #if FFESTR_F90
     case FFESTR_firstALLOCATE:
       ffestb_args.heap.len = FFESTR_firstlALLOCATE;
-      ffestb_args.heap.bad = FFEBAD_INVALID_ALLOCATE_STMT;
+      ffestb_args.heap.badname = "ALLOCATE";
       ffestb_args.heap.ctx = FFEEXPR_contextALLOCATE;
       ffesta_add_possible_exec_ ((ffelexHandler) ffestb_heap);
       break;
@@ -608,7 +608,7 @@ ffesta_second_ (ffelexToken t)
 
     case FFESTR_firstBACKSPACE:
       ffestb_args.beru.len = FFESTR_firstlBACKSPACE;
-      ffestb_args.beru.bad = FFEBAD_INVALID_BACKSPACE_STMT;
+      ffestb_args.beru.badname = "BACKSPACE";
       ffesta_add_possible_exec_ ((ffelexHandler) ffestb_beru);
       break;
 
@@ -621,7 +621,7 @@ ffesta_second_ (ffelexToken t)
 #if FFESTR_F90
     case FFESTR_firstDEALLOCATE:
       ffestb_args.heap.len = FFESTR_firstlDEALLOCATE;
-      ffestb_args.heap.bad = FFEBAD_INVALID_DEALLOCATE_STMT;
+      ffestb_args.heap.badname = "DEALLOCATE";
       ffestb_args.heap.ctx = FFEEXPR_contextDEALLOCATE;
       ffesta_add_possible_exec_ ((ffelexHandler) ffestb_heap);
       break;
@@ -630,14 +630,14 @@ ffesta_second_ (ffelexToken t)
 #if FFESTR_VXT
     case FFESTR_firstDECODE:
       ffestb_args.vxtcode.len = FFESTR_firstlDECODE;
-      ffestb_args.vxtcode.bad = FFEBAD_INVALID_DECODE_STMT;
+      ffestb_args.vxtcode.badname = "DECODE";
       ffesta_add_possible_exec_ ((ffelexHandler) ffestb_vxtcode);
       break;
 #endif
 
     case FFESTR_firstDIMENSION:
       ffestb_args.R524.len = FFESTR_firstlDIMENSION;
-      ffestb_args.R524.bad = FFEBAD_INVALID_DIMENSION_STMT;
+      ffestb_args.R524.badname = "DIMENSION";
       ffesta_add_possible_nonexec_ ((ffelexHandler) ffestb_R524);
       break;
 
@@ -668,7 +668,7 @@ ffesta_second_ (ffelexToken t)
 #if FFESTR_VXT
     case FFESTR_firstENCODE:
       ffestb_args.vxtcode.len = FFESTR_firstlENCODE;
-      ffestb_args.vxtcode.bad = FFEBAD_INVALID_ENCODE_STMT;
+      ffestb_args.vxtcode.badname = "ENCODE";
       ffesta_add_possible_exec_ ((ffelexHandler) ffestb_vxtcode);
       break;
 #endif
@@ -726,7 +726,7 @@ ffesta_second_ (ffelexToken t)
 
     case FFESTR_firstENDFILE:
       ffestb_args.beru.len = FFESTR_firstlENDFILE;
-      ffestb_args.beru.bad = FFEBAD_INVALID_ENDFILE_STMT;
+      ffestb_args.beru.badname = "ENDFILE";
       ffesta_add_possible_exec_ ((ffelexHandler) ffestb_beru);
       break;
 
@@ -818,14 +818,14 @@ ffesta_second_ (ffelexToken t)
 
     case FFESTR_firstENTRY:
       ffestb_args.dummy.len = FFESTR_firstlENTRY;
-      ffestb_args.dummy.bad = FFEBAD_INVALID_ENTRY_STMT;
+      ffestb_args.dummy.badname = "ENTRY";
       ffestb_args.dummy.is_subr = ffestc_is_entry_in_subr ();
       ffesta_add_possible_nonexec_ ((ffelexHandler) ffestb_dummy);
       break;
 
     case FFESTR_firstEXTERNAL:
       ffestb_args.varlist.len = FFESTR_firstlEXTERNAL;
-      ffestb_args.varlist.bad = FFEBAD_INVALID_EXTERNAL_STMT;
+      ffestb_args.varlist.badname = "EXTERNAL";
       ffesta_add_possible_nonexec_ ((ffelexHandler) ffestb_varlist);
       break;
 
@@ -841,7 +841,7 @@ ffesta_second_ (ffelexToken t)
 
     case FFESTR_firstFUNCTION:
       ffestb_args.dummy.len = FFESTR_firstlFUNCTION;
-      ffestb_args.dummy.bad = FFEBAD_INVALID_FUNCTION_STMT;
+      ffestb_args.dummy.badname = "FUNCTION";
       ffestb_args.dummy.is_subr = FALSE;
       ffesta_add_possible_nonexec_ ((ffelexHandler) ffestb_dummy);
       break;
@@ -871,14 +871,14 @@ ffesta_second_ (ffelexToken t)
 #if FFESTR_F90
     case FFESTR_firstINTENT:
       ffestb_args.varlist.len = FFESTR_firstlINTENT;
-      ffestb_args.varlist.bad = FFEBAD_INVALID_INTENT_STMT;
+      ffestb_args.varlist.badname = "INTENT";
       ffesta_add_possible_nonexec_ ((ffelexHandler) ffestb_varlist);
       break;
 #endif
 
     case FFESTR_firstINTRINSIC:
       ffestb_args.varlist.len = FFESTR_firstlINTRINSIC;
-      ffestb_args.varlist.bad = FFEBAD_INVALID_INTRINSIC_STMT;
+      ffestb_args.varlist.badname = "INTRINSIC";
       ffesta_add_possible_nonexec_ ((ffelexHandler) ffestb_varlist);
       break;
 
@@ -891,7 +891,7 @@ ffesta_second_ (ffelexToken t)
 #if FFESTR_F90
     case FFESTR_firstOPTIONAL:
       ffestb_args.varlist.len = FFESTR_firstlOPTIONAL;
-      ffestb_args.varlist.bad = FFEBAD_INVALID_OPTIONAL_STMT;
+      ffestb_args.varlist.badname = "OPTIONAL";
       ffesta_add_possible_nonexec_ ((ffelexHandler) ffestb_varlist);
       break;
 #endif
@@ -904,7 +904,7 @@ ffesta_second_ (ffelexToken t)
 #if FFESTR_F90
     case FFESTR_firstPOINTER:
       ffestb_args.dimlist.len = FFESTR_firstlPOINTER;
-      ffestb_args.dimlist.bad = FFEBAD_INVALID_POINTER_STMT;
+      ffestb_args.dimlist.badname = "POINTER";
       ffesta_add_possible_nonexec_ ((ffelexHandler) ffestb_dimlist);
       break;
 #endif
@@ -912,13 +912,13 @@ ffesta_second_ (ffelexToken t)
 #if HARD_F90
     case FFESTR_firstPRIVATE:
       ffestb_args.varlist.len = FFESTR_firstlPRIVATE;
-      ffestb_args.varlist.bad = FFEBAD_INVALID_ACCESS_STMT;
+      ffestb_args.varlist.badname = "ACCESS";
       ffesta_add_possible_nonexec_ ((ffelexHandler) ffestb_varlist);
       break;
 
     case FFESTR_firstPUBLIC:
       ffestb_args.varlist.len = FFESTR_firstlPUBLIC;
-      ffestb_args.varlist.bad = FFEBAD_INVALID_ACCESS_STMT;
+      ffestb_args.varlist.badname = "ACCESS";
       ffesta_add_possible_nonexec_ ((ffelexHandler) ffestb_varlist);
       break;
 #endif
@@ -931,7 +931,7 @@ ffesta_second_ (ffelexToken t)
 
     case FFESTR_firstREWIND:
       ffestb_args.beru.len = FFESTR_firstlREWIND;
-      ffestb_args.beru.bad = FFEBAD_INVALID_REWIND_STMT;
+      ffestb_args.beru.badname = "REWIND";
       ffesta_add_possible_exec_ ((ffelexHandler) ffestb_beru);
       break;
 
@@ -942,7 +942,7 @@ ffesta_second_ (ffelexToken t)
 
     case FFESTR_firstSUBROUTINE:
       ffestb_args.dummy.len = FFESTR_firstlSUBROUTINE;
-      ffestb_args.dummy.bad = FFEBAD_INVALID_SUBROUTINE_STMT;
+      ffestb_args.dummy.badname = "SUBROUTINE";
       ffestb_args.dummy.is_subr = TRUE;
       ffesta_add_possible_nonexec_ ((ffelexHandler) ffestb_dummy);
       break;
@@ -950,7 +950,7 @@ ffesta_second_ (ffelexToken t)
 #if FFESTR_F90
     case FFESTR_firstTARGET:
       ffestb_args.dimlist.len = FFESTR_firstlTARGET;
-      ffestb_args.dimlist.bad = FFEBAD_INVALID_TARGET_STMT;
+      ffestb_args.dimlist.badname = "TARGET";
       ffesta_add_possible_nonexec_ ((ffelexHandler) ffestb_dimlist);
       break;
 #endif
@@ -958,14 +958,14 @@ ffesta_second_ (ffelexToken t)
 #if FFESTR_VXT
     case FFESTR_firstUNLOCK:
       ffestb_args.beru.len = FFESTR_firstlUNLOCK;
-      ffestb_args.beru.bad = FFEBAD_INVALID_UNLOCK_STMT;
+      ffestb_args.beru.badname = "UNLOCK";
       ffesta_add_possible_exec_ ((ffelexHandler) ffestb_beru);
       break;
 #endif
 
     case FFESTR_firstVIRTUAL:
       ffestb_args.R524.len = FFESTR_firstlVIRTUAL;
-      ffestb_args.R524.bad = FFEBAD_INVALID_VIRTUAL_STMT;
+      ffestb_args.R524.badname = "VIRTUAL";
       ffesta_add_possible_nonexec_ ((ffelexHandler) ffestb_R524);
       break;
 
@@ -1355,30 +1355,17 @@ ffesta_first (ffelexToken t)
   switch (ffelex_token_type (t))
     {
     case FFELEX_typeSEMICOLON:
-      if (ffe_is_pedantic_not_90 ())
-	{
-	  ffebad_start (FFEBAD_SEMICOLON);
-	  ffebad_here (0, ffelex_token_where_line (t), ffelex_token_where_column (t));
-	  ffebad_finish ();
-	}
-      ffesta_last_token_was_semi_ = TRUE;
-      /* Fall through. */
     case FFELEX_typeEOS:
       ffesta_tokens[0] = ffelex_token_use (t);
-      if (ffesta_label_token == NULL)
-	return (ffelexHandler) ffesta_zero (t);
-
-      ffebad_start (FFEBAD_LABEL_WITHOUT_STMT);
-      ffebad_here (0, ffelex_token_where_line (ffesta_label_token),
-		   ffelex_token_where_column (ffesta_label_token));
-      ffebad_string (ffelex_token_text (ffesta_label_token));
-      ffebad_here (1, ffelex_token_where_line (t), ffelex_token_where_column (t));
-      ffebad_finish ();
-
-#if 0				/* Since we do CONTINUE, use label? */
-      ffelex_token_kill (ffesta_label_token);
-      ffesta_label_token = NULL;
-#endif
+      if (ffesta_label_token != NULL)
+	{
+	  ffebad_start (FFEBAD_LABEL_WITHOUT_STMT);
+	  ffebad_here (0, ffelex_token_where_line (ffesta_label_token),
+		       ffelex_token_where_column (ffesta_label_token));
+	  ffebad_string (ffelex_token_text (ffesta_label_token));
+	  ffebad_here (1, ffelex_token_where_line (t), ffelex_token_where_column (t));
+	  ffebad_finish ();
+	}
       return (ffelexHandler) ffesta_zero (t);
 
     case FFELEX_typeNAME:
@@ -1388,7 +1375,7 @@ ffesta_first (ffelexToken t)
       return (ffelexHandler) ffesta_second_;
 
     case FFELEX_typeNUMBER:
-      if (ffesta_last_token_was_semi_
+      if (ffesta_line_has_semicolons
 	  && !ffe_is_free_form ()
 	  && ffe_is_pedantic ())
 	{
@@ -1553,6 +1540,51 @@ ffesta_ffebad_1p (ffebad errnum, ffelexToken names_token, ffeTokenLength index,
     }
 }
 
+void
+ffesta_ffebad_1sp (ffebad errnum, char *s, ffelexToken names_token,
+		   ffeTokenLength index, ffelexToken next_token)
+{
+  ffewhereLine line;
+  ffewhereColumn col;
+
+  assert (index <= ffelex_token_length (names_token));
+
+  if (ffesta_ffebad_start (errnum))
+    {
+      ffebad_string (s);
+      if (index == ffelex_token_length (names_token))
+	{
+	  assert (next_token != NULL);
+	  line = ffelex_token_where_line (next_token);
+	  col = ffelex_token_where_column (next_token);
+	  ffebad_here (0, line, col);
+	}
+      else
+	{
+	  ffewhere_set_from_track (&line, &col,
+				   ffelex_token_where_line (names_token),
+				   ffelex_token_where_column (names_token),
+				   ffelex_token_wheretrack (names_token),
+				   index);
+	  ffebad_here (0, line, col);
+	  ffewhere_line_kill (line);
+	  ffewhere_column_kill (col);
+	}
+      ffebad_finish ();
+    }
+}
+
+void
+ffesta_ffebad_1st (ffebad errnum, char *s, ffelexToken t)
+{
+  if (ffesta_ffebad_start (errnum))
+    {
+      ffebad_string (s);
+      ffebad_here (0, ffelex_token_where_line (t), ffelex_token_where_column (t));
+      ffebad_finish ();
+    }
+}
+
 /* ffesta_ffebad_1t -- Issue diagnostic with one source token
 
    ffelexToken t;
@@ -1571,6 +1603,18 @@ ffesta_ffebad_1t (ffebad errnum, ffelexToken t)
     }
 }
 
+void
+ffesta_ffebad_2st (ffebad errnum, char *s, ffelexToken t1, ffelexToken t2)
+{
+  if (ffesta_ffebad_start (errnum))
+    {
+      ffebad_string (s);
+      ffebad_here (0, ffelex_token_where_line (t1), ffelex_token_where_column (t1));
+      ffebad_here (1, ffelex_token_where_line (t2), ffelex_token_where_column (t2));
+      ffebad_finish ();
+    }
+}
+
 /* ffesta_ffebad_2t -- Issue diagnostic with two source tokens
 
    ffelexToken t1, t2;
@@ -1752,16 +1796,18 @@ ffesta_zero (ffelexToken t)
   ffesta_reset_possibles_ ();
   ffesta_confirmed_current_ = FALSE;
 
-  if ((ffelex_token_type (t) == FFELEX_typeSEMICOLON)
-      && ffe_is_pedantic_not_90 ())
+  if (ffelex_token_type (t) == FFELEX_typeSEMICOLON)
     {
-      ffesta_last_token_was_semi_ = TRUE;
-      ffebad_start (FFEBAD_SEMICOLON);
-      ffebad_here (0, ffelex_token_where_line (t), ffelex_token_where_column (t));
-      ffebad_finish ();
+      ffesta_line_has_semicolons = TRUE;
+      if (ffe_is_pedantic_not_90 ())
+	{
+	  ffebad_start (FFEBAD_SEMICOLON);
+	  ffebad_here (0, ffelex_token_where_line (t), ffelex_token_where_column (t));
+	  ffebad_finish ();
+	}
     }
   else
-    ffesta_last_token_was_semi_ = FALSE;
+    ffesta_line_has_semicolons = FALSE;
 
   if (ffesta_label_token != NULL)
     {
