@@ -279,12 +279,6 @@ ffetarget_align (ffetargetAlign *updated_alignment,
   return min_pad;
 }
 
-/* ffetarget_character1 -- Convert token to a character constant
-
-   See prototype.
-
-   Token use count not affected overall.  */
-
 #if FFETARGET_okCHARACTER1
 bool
 ffetarget_character1 (ffetargetCharacter1 *val, ffelexToken character,
@@ -303,9 +297,7 @@ ffetarget_character1 (ffetargetCharacter1 *val, ffelexToken character,
 }
 
 #endif
-/* ffetarget_cmp_character1 -- Produce orderable comparison between two constants
-
-   See the prototype.
+/* Produce orderable comparison between two constants
 
    Compare lengths, if equal then use memcmp.  */
 
@@ -325,8 +317,6 @@ ffetarget_cmp_character1 (ffetargetCharacter1 l, ffetargetCharacter1 r)
 #endif
 /* ffetarget_concatenate_character1 -- Perform CONCAT op on two constants
 
-   See the prototype.
-
    Compare lengths, if equal then use memcmp.  */
 
 #if FFETARGET_okCHARACTER1
@@ -353,8 +343,6 @@ ffetarget_concatenate_character1 (ffetargetCharacter1 *res,
 #endif
 /* ffetarget_eq_character1 -- Perform relational comparison on char constants
 
-   See the prototype.
-
    Compare lengths, if equal then use memcmp.  */
 
 #if FFETARGET_okCHARACTER1
@@ -370,8 +358,6 @@ ffetarget_eq_character1 (bool *res, ffetargetCharacter1 l,
 #endif
 /* ffetarget_le_character1 -- Perform relational comparison on char constants
 
-   See the prototype.
-
    Compare lengths, if equal then use memcmp.  */
 
 #if FFETARGET_okCHARACTER1
@@ -387,8 +373,6 @@ ffetarget_le_character1 (bool *res, ffetargetCharacter1 l,
 #endif
 /* ffetarget_lt_character1 -- Perform relational comparison on char constants
 
-   See the prototype.
-
    Compare lengths, if equal then use memcmp.  */
 
 #if FFETARGET_okCHARACTER1
@@ -404,8 +388,6 @@ ffetarget_lt_character1 (bool *res, ffetargetCharacter1 l,
 #endif
 /* ffetarget_ge_character1 -- Perform relational comparison on char constants
 
-   See the prototype.
-
    Compare lengths, if equal then use memcmp.  */
 
 #if FFETARGET_okCHARACTER1
@@ -421,8 +403,6 @@ ffetarget_ge_character1 (bool *res, ffetargetCharacter1 l,
 #endif
 /* ffetarget_gt_character1 -- Perform relational comparison on char constants
 
-   See the prototype.
-
    Compare lengths, if equal then use memcmp.  */
 
 #if FFETARGET_okCHARACTER1
@@ -438,8 +418,6 @@ ffetarget_gt_character1 (bool *res, ffetargetCharacter1 l,
 #endif
 /* ffetarget_layout -- Do storage requirement analysis for entity
 
-   See the prototype.
-
    Return the alignment/modulo requirements along with the size, given the
    data type info and the number of elements an array (1 for a scalar).	 */
 
@@ -462,8 +440,10 @@ ffetarget_layout (char *error_text, ffetargetAlign *alignment,
   if (bt == FFEINFO_basictypeCHARACTER)
     {
       ok = ffetarget_offset_charsize (size, charsize, ffetype_size (type));
+#ifdef ffetarget_offset_overflow
       if (!ok)
 	ffetarget_offset_overflow (error_text);
+#endif
     }
   else
     *size = ffetype_size (type);
@@ -481,8 +461,6 @@ ffetarget_layout (char *error_text, ffetargetAlign *alignment,
 
 /* ffetarget_ne_character1 -- Perform relational comparison on char constants
 
-   See the prototype.
-
    Compare lengths, if equal then use memcmp.  */
 
 #if FFETARGET_okCHARACTER1
@@ -498,8 +476,6 @@ ffetarget_ne_character1 (bool *res, ffetargetCharacter1 l,
 #endif
 /* ffetarget_substr_character1 -- Perform SUBSTR op on three constants
 
-   See the prototype.
-
    Compare lengths, if equal then use memcmp.  */
 
 #if FFETARGET_okCHARACTER1
@@ -529,8 +505,6 @@ ffetarget_substr_character1 (ffetargetCharacter1 *res,
 /* ffetarget_cmp_hollerith -- Produce orderable comparison between two
    constants
 
-   See the prototype.
-
    Compare lengths, if equal then use memcmp.  */
 
 int
@@ -543,63 +517,149 @@ ffetarget_cmp_hollerith (ffetargetHollerith l, ffetargetHollerith r)
   return memcmp (l.text, r.text, l.length);
 }
 
-/* ffetarget_convert_any_hollerith_ -- Convert hollerith constant
+ffebad
+ffetarget_convert_any_character1_ (char *res, size_t size,
+				   ffetargetCharacter1 l)
+{
+  if (size <= l.length)
+    {
+      char *p;
+      ffetargetCharacterSize i;
+      
+      memcpy (res, l.text, size);
+      for (p = &l.text[0] + size, i = l.length - size;
+	   i > 0;
+	   ++p, --i)
+	if (*p != ' ')
+	  return FFEBAD_TRUNCATING_CHARACTER;
+    }
+  else
+    {
+      memcpy (res, l.text, size);
+      memset (res + l.length, ' ', size - l.length);
+    }
 
-   See prototype.
-
-   Token use count not affected overall.  */
+  return FFEBAD;
+}
 
 ffebad
 ffetarget_convert_any_hollerith_ (char *res, size_t size,
 				  ffetargetHollerith l)
 {
-  memcpy (res, l.text, size);
-  if (size > l.length)
-    memset (res + l.length, 0, size - l.length);
+  if (size <= l.length)
+    {
+      char *p;
+      ffetargetCharacterSize i;
+      
+      memcpy (res, l.text, size);
+      for (p = &l.text[0] + size, i = l.length - size;
+	   i > 0;
+	   ++p, --i)
+	if (*p != ' ')
+	  return FFEBAD_TRUNCATING_HOLLERITH;
+    }
+  else
+    {
+      memcpy (res, l.text, size);
+      memset (res + l.length, ' ', size - l.length);
+    }
 
   return FFEBAD;
 }
 
-/* ffetarget_convert_any_typeless_ -- Convert typeless constant
-
-   See prototype.
-
-   Token use count not affected overall.  */
-
 ffebad
 ffetarget_convert_any_typeless_ (char *res, size_t size,
 				 ffetargetTypeless l)
 {
-  memcpy (res, &l.bytes[ARRAY_SIZE (l.bytes) - size], size);
+  unsigned long long int l1;
+  unsigned long int l2;
+  unsigned int l3;
+  unsigned short int l4;
+  unsigned char l5;
+  int size_of;
+  char *p;
 
+  if (size >= sizeof (l1))
+    {
+      l1 = l;
+      p = (char *) &l1;
+      size_of = sizeof (l1);
+    }
+  else if (size >= sizeof (l2))
+    {
+      l2 = l;
+      p = (char *) &l2;
+      size_of = sizeof (l2);
+      l1 = l2;
+    }
+  else if (size >= sizeof (l3))
+    {
+      l3 = l;
+      p = (char *) &l3;
+      size_of = sizeof (l3);
+      l1 = l3;
+    }
+  else if (size >= sizeof (l4))
+    {
+      l4 = l;
+      p = (char *) &l4;
+      size_of = sizeof (l4);
+      l1 = l4;
+    }
+  else if (size >= sizeof (l5))
+    {
+      l5 = l;
+      p = (char *) &l5;
+      size_of = sizeof (l5);
+      l1 = l5;
+    }
+  else
+    {
+      assert ("stumped by conversion from typeless!" == NULL);
+      abort ();
+    }
+
+  if (size <= size_of)
+    {
+      int i = size_of - size;
+
+      memcpy (res, p + i, size);
+      for (; i > 0; ++p, --i)
+	if (*p != '\0')
+	  return FFEBAD_TRUNCATING_TYPELESS;
+    }
+  else
+    {
+      int i = size - size_of;
+
+      memset (res, 0, i);
+      memcpy (res + i, p, size_of);
+    }
+
+  if (l1 != l)
+    return FFEBAD_TRUNCATING_TYPELESS;
   return FFEBAD;
 }
 
-/* ffetarget_convert_character1_character1 -- Convert between different lengths
-
-   See the prototype.
-
-   Compare lengths, if equal then use memcmp.  */
-
 #if FFETARGET_okCHARACTER1
 ffebad
 ffetarget_convert_character1_character1 (ffetargetCharacter1 *res,
-					 ffetargetCharacterSize res_size,
+					 ffetargetCharacterSize size,
 					 ffetargetCharacter1 l,
 					 mallocPool pool)
 {
-  res->length = res_size;
-  if (res_size == 0)
+  res->length = size;
+  if (size == 0)
     res->text = NULL;
   else
     {
-      res->text = malloc_new_kp (pool, "FFETARGET cvt char1", res_size);
-      if (res_size <= l.length)
-	memcpy (res->text, l.text, res_size);
+      res->text = malloc_new_kp (pool, "FFETARGET cvt char1", size);
+      if (size <= l.length)
+	memcpy (res->text, l.text, size);
       else
 	{
 	  memcpy (res->text, l.text, l.length);
-	  memset (res->text + l.length, ' ', res_size - l.length);
+	  memset (res->text + l.length, ' ', size - l.length);
 	}
     }
 
@@ -607,57 +667,209 @@ ffetarget_convert_character1_character1 (ffetargetCharacter1 *res,
 }
 
 #endif
-/* ffetarget_convert_character1_hollerith -- Raw conversion.  */
-
 #if FFETARGET_okCHARACTER1
 ffebad
 ffetarget_convert_character1_hollerith (ffetargetCharacter1 *res,
-					ffetargetCharacterSize res_size,
-				      ffetargetHollerith l, mallocPool pool)
+					ffetargetCharacterSize size,
+					ffetargetHollerith l, mallocPool pool)
 {
-  res->length = res_size;
-  if (res_size == 0)
+  res->length = size;
+  if (size == 0)
     res->text = NULL;
   else
     {
-      res->text = malloc_new_kp (pool, "FFETARGET cvt char1", res_size);
-      if (res_size <= l.length)
-	memcpy (res->text, l.text, res_size);
+      res->text = malloc_new_kp (pool, "FFETARGET cvt char1", size);
+      if (size <= l.length)
+	{
+	  char *p;
+	  ffetargetCharacterSize i;
+
+	  memcpy (res->text, l.text, size);
+	  for (p = &l.text[0] + size, i = l.length - size;
+	       i > 0;
+	       ++p, --i)
+	    if (*p != ' ')
+	      return FFEBAD_TRUNCATING_HOLLERITH;
+	}
       else
 	{
 	  memcpy (res->text, l.text, l.length);
-	  memset (res->text + l.length, 0, res_size - l.length);
+	  memset (res->text + l.length, ' ', size - l.length);
 	}
     }
 
   return FFEBAD;
 }
 
+#endif
+/* ffetarget_convert_character1_integer1 -- Raw conversion.  */
+
+#if FFETARGET_okCHARACTER1
+ffebad
+ffetarget_convert_character1_integer1 (ffetargetCharacter1 *res,
+				       ffetargetCharacterSize size,
+				       ffetargetInteger1 l, mallocPool pool)
+{
+  unsigned long long int l1;
+  unsigned long int l2;
+  unsigned int l3;
+  unsigned short int l4;
+  unsigned char l5;
+  int size_of;
+  char *p;
+
+  if (size >= sizeof (l1))
+    {
+      l1 = l;
+      p = (char *) &l1;
+      size_of = sizeof (l1);
+    }
+  else if (size >= sizeof (l2))
+    {
+      l2 = l;
+      p = (char *) &l2;
+      size_of = sizeof (l2);
+      l1 = l2;
+    }
+  else if (size >= sizeof (l3))
+    {
+      l3 = l;
+      p = (char *) &l3;
+      size_of = sizeof (l3);
+      l1 = l3;
+    }
+  else if (size >= sizeof (l4))
+    {
+      l4 = l;
+      p = (char *) &l4;
+      size_of = sizeof (l4);
+      l1 = l4;
+    }
+  else if (size >= sizeof (l5))
+    {
+      l5 = l;
+      p = (char *) &l5;
+      size_of = sizeof (l5);
+      l1 = l5;
+    }
+  else
+    {
+      assert ("stumped by conversion from integer1!" == NULL);
+      abort ();
+    }
+
+  res->length = size;
+  if (size == 0)
+    res->text = NULL;
+  else
+    {
+      res->text = malloc_new_kp (pool, "FFETARGET cvt char1", size);
+      if (size <= size_of)
+	{
+	  int i = size_of - size;
+
+	  memcpy (res->text, p + i, size);
+	  for (; i > 0; ++p, --i)
+	    if (*p != 0)
+	      return FFEBAD_TRUNCATING_NUMERIC;
+	}
+      else
+	{
+	  int i = size - size_of;
+
+	  memset (res->text, 0, i);
+	  memcpy (res->text + i, p, size_of);
+	}
+    }
+
+  if (l1 != l)
+    return FFEBAD_TRUNCATING_NUMERIC;
+  return FFEBAD;
+}
+
 #endif
 /* ffetarget_convert_character1_typeless -- Raw conversion.  */
 
 #if FFETARGET_okCHARACTER1
 ffebad
 ffetarget_convert_character1_typeless (ffetargetCharacter1 *res,
-				       ffetargetCharacterSize res_size,
+				       ffetargetCharacterSize size,
 				       ffetargetTypeless l, mallocPool pool)
 {
-  res->length = res_size;
-  if (res_size == 0)
+  unsigned long long int l1;
+  unsigned long int l2;
+  unsigned int l3;
+  unsigned short int l4;
+  unsigned char l5;
+  int size_of;
+  char *p;
+
+  if (size >= sizeof (l1))
+    {
+      l1 = l;
+      p = (char *) &l1;
+      size_of = sizeof (l1);
+    }
+  else if (size >= sizeof (l2))
+    {
+      l2 = l;
+      p = (char *) &l2;
+      size_of = sizeof (l2);
+      l1 = l2;
+    }
+  else if (size >= sizeof (l3))
+    {
+      l3 = l;
+      p = (char *) &l3;
+      size_of = sizeof (l3);
+      l1 = l3;
+    }
+  else if (size >= sizeof (l4))
+    {
+      l4 = l;
+      p = (char *) &l4;
+      size_of = sizeof (l4);
+      l1 = l4;
+    }
+  else if (size >= sizeof (l5))
+    {
+      l5 = l;
+      p = (char *) &l5;
+      size_of = sizeof (l5);
+      l1 = l5;
+    }
+  else
+    {
+      assert ("stumped by conversion from typeless!" == NULL);
+      abort ();
+    }
+
+  res->length = size;
+  if (size == 0)
     res->text = NULL;
   else
     {
-      res->text = malloc_new_kp (pool, "FFETARGET cvt char1", res_size);
-      if (res_size <= ffetype_size_typeless ())
-	memcpy (res->text, l.bytes, res_size);
+      res->text = malloc_new_kp (pool, "FFETARGET cvt char1", size);
+      if (size <= size_of)
+	{
+	  int i = size_of - size;
+
+	  memcpy (res->text, p + i, size);
+	  for (; i > 0; ++p, --i)
+	    if (*p != 0)
+	      return FFEBAD_TRUNCATING_TYPELESS;
+	}
       else
 	{
-	  memcpy (res->text, l.bytes, ffetype_size_typeless ());
-	  memset (res->text + ffetype_size_typeless (), 0,
-		  res_size - ffetype_size_typeless ());
+	  int i = size - size_of;
+
+	  memset (res->text, 0, i);
+	  memcpy (res->text + i, p, size_of);
 	}
     }
 
+  if (l1 != l)
+    return FFEBAD_TRUNCATING_TYPELESS;
   return FFEBAD;
 }
 
@@ -795,8 +1007,6 @@ ffetarget_hollerith (ffetargetHollerith *val, ffelexToken integer,
 
 /* ffetarget_integer_bad_magical -- Complain about a magical number
 
-   See the prototype.
-
    Just calls ffebad with the arguments.  */
 
 void
@@ -809,8 +1019,6 @@ ffetarget_integer_bad_magical (ffelexToken t)
 
 /* ffetarget_integer_bad_magical_binary -- Complain about a magical number
 
-   See the prototype.
-
    Just calls ffebad with the arguments.  */
 
 void
@@ -828,8 +1036,6 @@ ffetarget_integer_bad_magical_binary (ffelexToken integer,
 /* ffetarget_integer_bad_magical_precedence -- Complain about a magical
 						   number
 
-   See the prototype.
-
    Just calls ffebad with the arguments.  */
 
 void
@@ -849,8 +1055,6 @@ ffetarget_integer_bad_magical_precedence (ffelexToken integer,
 
 /* ffetarget_integer_bad_magical_precedence_binary -- Complain...
 
-   See the prototype.
-
    Just calls ffebad with the arguments.  */
 
 void
@@ -1517,38 +1721,20 @@ void
 ffetarget_print_binary (FILE *f, ffetargetTypeless value)
 {
   char *p;
-  char *s;
-  char c;
-  int i;
-  int d;
-  bool bits[ARRAY_SIZE (value.bytes) * CHAR_BIT];
-  char digits[ARRAY_SIZE (bits) + 1];
+  char digits[sizeof (value) * CHAR_BIT + 1];
 
-  for (d = 0; d < ARRAY_SIZE (value.bytes); ++d)
-    {
-      c = value.bytes[d];
-      for (i = 0; i < CHAR_BIT; ++i)
-	{
-	  bits[d * CHAR_BIT + i] = (c & (1 << (CHAR_BIT - i - 1))) != 0
-	    ? TRUE : FALSE;
-	}
-    }
-
-  /* Build digit string. */
+  if (f == NULL)
+    f = stdout;
 
   p = &digits[ARRAY_SIZE (digits) - 1];
   *p = '\0';
-  s = p - 1;			/* Output at least one zero. */
-  for (d = ARRAY_SIZE (bits) - 1; p != &digits[0]; --d)
+  do
     {
-      *--p = bits[d] ? '1' : '0';
-      if (*p != '0')
-	s = p;
-    };
+      *--p = (value & 1) + '0';
+      value >>= 1;
+    } while (value == 0);
 
-  /* Output string starting with first nonzero digit. */
-
-  fputs (s, stdout);
+  fputs (p, f);
 }
 
 /* ffetarget_print_character1 -- Output character string
@@ -1594,41 +1780,20 @@ void
 ffetarget_print_octal (FILE *f, ffetargetTypeless value)
 {
   char *p;
-  char *s;
-  char c;
-  int i;
-  int d;
-  bool bits[ARRAY_SIZE (value.bytes) * CHAR_BIT + 2];	/* 2 extra in front. */
-  char digits[ARRAY_SIZE (bits) / 3 + 1];
+  char digits[sizeof (value) * CHAR_BIT / 3 + 1];
 
-  for (d = 0; d < ARRAY_SIZE (value.bytes); ++d)
-    {
-      c = value.bytes[d];
-      for (i = 0; i < CHAR_BIT; ++i)
-	{
-	  bits[d * CHAR_BIT + i + 2] = (c & (1 << (CHAR_BIT - i - 1))) != 0
-	    ? TRUE : FALSE;
-	}
-    }
-  bits[0] = FALSE;
-  bits[1] = FALSE;
+  if (f == NULL)
+    f = stdout;
 
-  /* Build digit string. */
-
-  p = &digits[ARRAY_SIZE (digits) - 1];
+  p = &digits[ARRAY_SIZE (digits) - 3];
   *p = '\0';
-  s = p - 1;			/* Output at least one zero. */
-  for (d = ARRAY_SIZE (bits) - 1; p != &digits[0]; d -= 3)
+  do
     {
-      c = (bits[d - 2] ? 4 : 0) | (bits[d - 1] ? 2 : 0) | (bits[d] ? 1 : 0);
-      *--p = c + '0';
-      if (*p != '0')
-	s = p;
-    };
+      *--p = (value & 3) + '0';
+      value >>= 3;
+    } while (value == 0);
 
-  /* Output string starting with first nonzero digit. */
-
-  fputs (s, stdout);
+  fputs (p, f);
 }
 
 /* ffetarget_print_hex -- Output typeless hex integer
@@ -1640,43 +1805,21 @@ void
 ffetarget_print_hex (FILE *f, ffetargetTypeless value)
 {
   char *p;
-  char *s;
-  char c;
-  int i;
-  int d;
-  bool bits[ARRAY_SIZE (value.bytes) * CHAR_BIT + 3];	/* 3 extra in front. */
-  char digits[ARRAY_SIZE (bits) / 4 + 1];
+  char digits[sizeof (value) * CHAR_BIT / 4 + 1];
+  static char hexdigits[16] = "0123456789ABCDEF";
 
-  for (d = 0; d < ARRAY_SIZE (value.bytes); ++d)
-    {
-      c = value.bytes[d];
-      for (i = 0; i < CHAR_BIT; ++i)
-	{
-	  bits[d * CHAR_BIT + i + 3] = (c & (1 << (CHAR_BIT - i - 1))) != 0
-	    ? TRUE : FALSE;
-	}
-    }
-  bits[0] = FALSE;
-  bits[1] = FALSE;
-  bits[2] = FALSE;
+  if (f == NULL)
+    f = stdout;
 
-  /* Build digit string. */
-
-  p = &digits[ARRAY_SIZE (digits) - 1];
+  p = &digits[ARRAY_SIZE (digits) - 3];
   *p = '\0';
-  s = p - 1;			/* Output at least one zero. */
-  for (d = ARRAY_SIZE (bits) - 1; p != &digits[0]; d -= 4)
+  do
     {
-      c = (bits[d - 3] ? 8 : 0) | (bits[d - 2] ? 4 : 0) | (bits[d - 1] ? 2 : 0)
-	| (bits[d] ? 1 : 0);
-      *--p = (c > 9) ? c + 'A' - 10 : c + '0';
-      if (*p != '0')
-	s = p;
-    };
+      *--p = hexdigits[value & 4];
+      value >>= 4;
+    } while (value == 0);
 
-  /* Output string starting with first nonzero digit. */
-
-  fputs (s, stdout);
+  fputs (p, f);
 }
 
 /* ffetarget_real1 -- Convert token to a single-precision real number
@@ -1842,59 +1985,28 @@ ffetarget_real2 (ffetargetReal2 *value, ffelexToken integer,
 }
 
 #endif
-/* ffetarget_typeless_binary -- Convert token to typeless binary integer
-
-   ffetargetTypeless x;
-   ffetarget_typeless_binary(&x,token);
-
-   Token use count not affected overall.  */
-
 bool
-ffetarget_typeless_binary (ffetargetTypeless *value, ffelexToken token)
+ffetarget_typeless_binary (ffetargetTypeless *xvalue, ffelexToken token)
 {
   char *p;
   char c;
-  bool bad_digit;
-  int i;
-  int d;
-  bool bits[ARRAY_SIZE (value->bytes) * CHAR_BIT];
+  ffetargetTypeless value = 0;
+  ffetargetTypeless new_value = 0;
+  bool bad_digit = FALSE;
+  bool overflow = FALSE;
 
   p = ffelex_token_text (token);
 
-  for (i = 0; i < ARRAY_SIZE (bits); ++i)
-    bits[i] = FALSE;
-
-  /* Skip past leading zeros. */
-
-  while (((c = *p) != '\0') && (c == '0'))
-    ++p;
-
-  /* Interpret rest of number. */
-
-  bad_digit = FALSE;
-  while (c != '\0')
+  for (c = *p; c != '\0'; c = *++p)
     {
-      if (!isdigit (c) || (c >= '2'))
-	{
-	  c = '0';
-	  bad_digit = TRUE;
-	}
-      d = c - '0';
-      for (i = 1; i < ARRAY_SIZE (bits); ++i)
-	bits[i - 1] = bits[i];
-      bits[ARRAY_SIZE (bits) - 1] = (d & 1) ? TRUE : FALSE;
-      c = *(++p);
-    };
-
-  for (d = 0; d < ARRAY_SIZE (value->bytes); ++d)
-    {
-      c = 0;
-      for (i = 0; i < CHAR_BIT; ++i)
-	{
-	  if (bits[d * CHAR_BIT + i])
-	    c |= 1 << (CHAR_BIT - i - 1);
-	}
-      value->bytes[d] = c;
+      new_value <<= 1;
+      if ((new_value >> 1) != value)
+	overflow = TRUE;
+      if (isdigit (c))
+	new_value += c - '0';
+      else
+	bad_digit = TRUE;
+      value = new_value;
     }
 
   if (bad_digit)
@@ -1904,90 +2016,41 @@ ffetarget_typeless_binary (ffetargetTypeless *value, ffelexToken token)
 		   ffelex_token_where_column (token));
       ffebad_finish ();
     }
+  else if (overflow)
+    {
+      ffebad_start (FFEBAD_TYPELESS_OVERFLOW);
+      ffebad_here (0, ffelex_token_where_line (token),
+		   ffelex_token_where_column (token));
+      ffebad_finish ();
+    }
 
-  return !bad_digit;
+  *xvalue = value;
+
+  return !bad_digit && !overflow;
 }
 
-/* ffetarget_typeless_octal -- Convert token to typeless octal integer
-
-   ffetargetTypeless x;
-   ffetarget_typeless_octal(&x,token);
-
-   Token use count not affected overall.  */
-
 bool
-ffetarget_typeless_octal (ffetargetTypeless *value, ffelexToken token)
+ffetarget_typeless_octal (ffetargetTypeless *xvalue, ffelexToken token)
 {
   char *p;
   char c;
-  bool bad_digit;
-  int i;
-  int d;
-  bool bits[ARRAY_SIZE (value->bytes) * CHAR_BIT];
+  ffetargetTypeless value = 0;
+  ffetargetTypeless new_value = 0;
+  bool bad_digit = FALSE;
+  bool overflow = FALSE;
 
   p = ffelex_token_text (token);
 
-  for (i = 0; i < ARRAY_SIZE (bits); ++i)
-    bits[i] = FALSE;
-
-  /* Skip past leading zeros. */
-
-  while (((c = *p) != '\0') && (c == '0'))
-    ++p;
-
-  /* Interpret rest of number. */
-
-  bad_digit = FALSE;
-  while (c != '\0')
+  for (c = *p; c != '\0'; c = *++p)
     {
-      if (!isdigit (c) || (c >= '8'))
-	{
-	  c = '0';
-	  bad_digit = TRUE;
-	}
-      d = c - '0';
-      switch (ARRAY_SIZE (bits) % 3)
-	{
-	case 0:
-	  i = 3;
-	  break;
-
-	case 1:
-	  i = 4;
-	  bits[0] = bits[3];
-	  break;
-
-	case 2:
-	  i = 5;
-	  bits[0] = bits[3];
-	  bits[1] = bits[4];
-	  break;
-
-	default:
-	  assert ("huh?" == NULL);
-	  break;
-	}
-      for (; i < ARRAY_SIZE (bits); i += 3)
-	{
-	  bits[i - 3] = bits[i];
-	  bits[i - 2] = bits[i + 1];
-	  bits[i - 1] = bits[i + 2];
-	}
-      bits[ARRAY_SIZE (bits) - 3] = (d & 4) ? TRUE : FALSE;
-      bits[ARRAY_SIZE (bits) - 2] = (d & 2) ? TRUE : FALSE;
-      bits[ARRAY_SIZE (bits) - 1] = (d & 1) ? TRUE : FALSE;
-      c = *(++p);
-    };
-
-  for (d = 0; d < ARRAY_SIZE (value->bytes); ++d)
-    {
-      c = 0;
-      for (i = 0; i < CHAR_BIT; ++i)
-	{
-	  if (bits[d * CHAR_BIT + i])
-	    c |= 1 << (CHAR_BIT - i - 1);
-	}
-      value->bytes[d] = c;
+      new_value <<= 3;
+      if ((new_value >> 3) != value)
+	overflow = TRUE;
+      if (isdigit (c))
+	new_value += c - '0';
+      else
+	bad_digit = TRUE;
+      value = new_value;
     }
 
   if (bad_digit)
@@ -1997,104 +2060,45 @@ ffetarget_typeless_octal (ffetargetTypeless *value, ffelexToken token)
 		   ffelex_token_where_column (token));
       ffebad_finish ();
     }
+  else if (overflow)
+    {
+      ffebad_start (FFEBAD_TYPELESS_OVERFLOW);
+      ffebad_here (0, ffelex_token_where_line (token),
+		   ffelex_token_where_column (token));
+      ffebad_finish ();
+    }
 
-  return !bad_digit;
+  *xvalue = value;
+
+  return !bad_digit && !overflow;
 }
 
-/* ffetarget_typeless_hex -- Convert token to typeless hexadecimal integer
-
-   ffetargetTypeless x;
-   ffetarget_typeless_hex(&x,token);
-
-   Token use count not affected overall.  */
-
 bool
-ffetarget_typeless_hex (ffetargetTypeless *value, ffelexToken token)
+ffetarget_typeless_hex (ffetargetTypeless *xvalue, ffelexToken token)
 {
   char *p;
   char c;
-  bool bad_digit;
-  int i;
-  int d;
-  bool bits[ARRAY_SIZE (value->bytes) * CHAR_BIT];
+  ffetargetTypeless value = 0;
+  ffetargetTypeless new_value = 0;
+  bool bad_digit = FALSE;
+  bool overflow = FALSE;
 
   p = ffelex_token_text (token);
 
-  for (i = 0; i < ARRAY_SIZE (bits); ++i)
-    bits[i] = FALSE;
-
-  /* Skip past leading zeros. */
-
-  while (((c = *p) != '\0') && (c == '0'))
-    ++p;
-
-  /* Interpret rest of number. */
-
-  bad_digit = FALSE;
-  while (c != '\0')
+  for (c = *p; c != '\0'; c = *++p)
     {
-      if (!isxdigit (c))
-	{
-	  c = '0';
-	  bad_digit = TRUE;
-	}
+      new_value <<= 4;
+      if ((new_value >> 4) != value)
+	overflow = TRUE;
       if (isdigit (c))
-	d = c - '0';
-      else if (isupper (c))
-	d = c - 'A' + 10;
+	new_value += c - '0';
+      else if ((c >= 'A') && (c <= 'F'))
+	new_value += c - 'A' + 10;
+      else if ((c >= 'a') && (c <= 'f'))
+	new_value += c - 'a' + 10;
       else
-	d = c - 'a' + 10;
-      switch (ARRAY_SIZE (bits) % 4)
-	{
-	case 0:
-	  i = 4;
-	  break;
-
-	case 1:
-	  i = 5;
-	  bits[0] = bits[4];
-	  break;
-
-	case 2:
-	  i = 6;
-	  bits[0] = bits[4];
-	  bits[1] = bits[5];
-	  break;
-
-	case 3:
-	  i = 7;
-	  bits[0] = bits[4];
-	  bits[1] = bits[5];
-	  bits[2] = bits[6];
-	  break;
-
-	default:
-	  assert ("huh?" == NULL);
-	  break;
-	}
-      for (; i < ARRAY_SIZE (bits); i += 4)
-	{
-	  bits[i - 4] = bits[i];
-	  bits[i - 3] = bits[i + 1];
-	  bits[i - 2] = bits[i + 2];
-	  bits[i - 1] = bits[i + 3];
-	}
-      bits[ARRAY_SIZE (bits) - 4] = (d & 8) ? TRUE : FALSE;
-      bits[ARRAY_SIZE (bits) - 3] = (d & 4) ? TRUE : FALSE;
-      bits[ARRAY_SIZE (bits) - 2] = (d & 2) ? TRUE : FALSE;
-      bits[ARRAY_SIZE (bits) - 1] = (d & 1) ? TRUE : FALSE;
-      c = *(++p);
-    };
-
-  for (d = 0; d < ARRAY_SIZE (value->bytes); ++d)
-    {
-      c = 0;
-      for (i = 0; i < CHAR_BIT; ++i)
-	{
-	  if (bits[d * CHAR_BIT + i])
-	    c |= 1 << (CHAR_BIT - i - 1);
-	}
-      value->bytes[d] = c;
+	bad_digit = TRUE;
+      value = new_value;
     }
 
   if (bad_digit)
@@ -2104,8 +2108,17 @@ ffetarget_typeless_hex (ffetargetTypeless *value, ffelexToken token)
 		   ffelex_token_where_column (token));
       ffebad_finish ();
     }
+  else if (overflow)
+    {
+      ffebad_start (FFEBAD_TYPELESS_OVERFLOW);
+      ffebad_here (0, ffelex_token_where_line (token),
+		   ffelex_token_where_column (token));
+      ffebad_finish ();
+    }
 
-  return !bad_digit;
+  *xvalue = value;
+
+  return !bad_digit && !overflow;
 }
 
 /* This is like memcpy.	 It is needed because some systems' header files
