@@ -60,6 +60,12 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 #ifndef FFETARGET_defaultIS_AUTOMATIC
 #define FFETARGET_defaultIS_AUTOMATIC 1
 #endif
+#ifndef FFETARGET_defaultIS_BACKSLASH
+#define FFETARGET_defaultIS_BACKSLASH 1
+#endif
+#ifndef FFETARGET_defaultIS_INIT_LOCAL_ZERO
+#define FFETARGET_defaultIS_INIT_LOCAL_ZERO 0
+#endif
 #ifndef FFETARGET_defaultIS_DOLLAR_OK
 #define FFETARGET_defaultIS_DOLLAR_OK 0
 #endif
@@ -79,7 +85,10 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 #define FFETARGET_defaultIS_UGLY 0
 #endif
 #ifndef FFETARGET_defaultIS_UGLY_ARGS
-#define FFETARGET_defaultIS_UGLY_ARGS 0
+#define FFETARGET_defaultIS_UGLY_ARGS 1
+#endif
+#ifndef FFETARGET_defaultIS_UGLY_INIT
+#define FFETARGET_defaultIS_UGLY_INIT 1
 #endif
 #ifndef FFETARGET_defaultIS_VXT_NOT_90
 #define FFETARGET_defaultIS_VXT_NOT_90 0
@@ -200,10 +209,6 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 #define FFETARGET_offsetNONE 0	/* Not used by FFE, for backend if needed. */
 #endif
 
-#ifndef FFETARGET_sizeTYPELESS
-#define FFETARGET_sizeTYPELESS 40	/* Max # bytes used by any number. */
-#endif
-
 #define FFETARGET_okINTEGER1 1
 #define FFETARGET_okINTEGER2 0
 #define FFETARGET_okINTEGER3 0
@@ -498,11 +503,7 @@ typedef ? ffetargetCharacter8;
 typedef ? ffetargetCharacterUnit8;
 #endif
 
-struct _ffetarget_typeless_
-  {
-    unsigned char bytes[FFETARGET_sizeTYPELESS];
-  };
-typedef struct _ffetarget_typeless_ ffetargetTypeless;
+typedef unsigned long long int ffetargetTypeless;
 
 struct _ffetarget_hollerith_
   {
@@ -575,6 +576,10 @@ ffebad ffetarget_convert_character1_hollerith (ffetargetCharacter1 *res,
 					    ffetargetCharacterSize res_size,
 					       ffetargetHollerith l,
 					       mallocPool pool);
+ffebad ffetarget_convert_character1_integer1 (ffetargetCharacter1 *res,
+					      ffetargetCharacterSize res_size,
+					      ffetargetInteger1 l,
+					      mallocPool pool);
 ffebad ffetarget_convert_character1_typeless (ffetargetCharacter1 *res,
 					    ffetargetCharacterSize res_size,
 					      ffetargetTypeless l,
@@ -602,6 +607,8 @@ int ffetarget_cmp_hollerith (ffetargetHollerith l, ffetargetHollerith r);
 bool ffetarget_hollerith (ffetargetHollerith *val, ffelexToken hollerith,
 			  mallocPool pool);
 int ffetarget_cmp_typeless (ffetargetTypeless l, ffetargetTypeless r);
+ffebad ffetarget_convert_any_character1_ (char *res, size_t size,
+					  ffetargetCharacter1 l);
 ffebad ffetarget_convert_any_hollerith_ (char *res, size_t size,
 					 ffetargetHollerith l);
 ffebad ffetarget_convert_any_typeless_ (char *res, size_t size,
@@ -868,7 +875,9 @@ void *ffetarget_memcpy_ (void *dst, void *src, size_t len);
 #define ffetarget_cmp_real2(l,r) memcmp (&(l), &(r), sizeof(l))
 #define ffetarget_cmp_real3(l,r) memcmp (&(l), &(r), sizeof(l))
 #define ffetarget_cmp_typeless(l,r) \
-  memcmp (&(l).bytes[0], &(r).bytes[0], ARRAY_SIZE ((l).bytes))
+  memcmp (&(l), &(r), sizeof ((l)))
+#define ffetarget_convert_complex1_character1(res,l) \
+  ffetarget_convert_any_character1_ ((char *) (res), sizeof(*(res)), l)
 #define ffetarget_convert_complex1_hollerith(res,l) \
   ffetarget_convert_any_hollerith_ ((char *) (res), sizeof(*(res)), l)
 #define ffetarget_convert_complex1_typeless(res,l) \
@@ -916,6 +925,8 @@ void *ffetarget_memcpy_ (void *dst, void *src, size_t len);
 #define ffetarget_convert_complex1_real2(res,l) \
   ((res)->real = (l), (res)->imaginary = 0, FFEBAD)
 #endif
+#define ffetarget_convert_complex2_character1(res,l) \
+  ffetarget_convert_any_character1_ ((char *) (res), sizeof(*(res)), l)
 #define ffetarget_convert_complex2_hollerith(res,l) \
   ffetarget_convert_any_hollerith_ ((char *) (res), sizeof(*(res)), l)
 #define ffetarget_convert_complex2_typeless(res,l) \
@@ -963,6 +974,8 @@ void *ffetarget_memcpy_ (void *dst, void *src, size_t len);
 #define ffetarget_convert_complex2_real2(res,l) \
   ((res)->real = (l), (res)->imaginary = 0, FFEBAD)
 #endif
+#define ffetarget_convert_logical1_character1(res,l) \
+  ffetarget_convert_any_character1_ ((char *) (res), sizeof(*(res)), l)
 #define ffetarget_convert_logical1_hollerith(res,l) \
   ffetarget_convert_any_hollerith_ ((char *) (res), sizeof(*(res)), l)
 #define ffetarget_convert_logical1_typeless(res,l) \
@@ -972,6 +985,8 @@ void *ffetarget_memcpy_ (void *dst, void *src, size_t len);
 #define ffetarget_convert_logical1_integer1(res,l) (*(res) = (l), FFEBAD)
 #define ffetarget_convert_logical1_integer2(res,l) (*(res) = (l), FFEBAD)
 #define ffetarget_convert_logical1_integer3(res,l) (*(res) = (l), FFEBAD)
+#define ffetarget_convert_logical2_character1(res,l) \
+  ffetarget_convert_any_character1_ ((char *) (res), sizeof(*(res)), l)
 #define ffetarget_convert_logical2_hollerith(res,l) \
   ffetarget_convert_any_hollerith_ ((char *) (res), sizeof(*(res)), l)
 #define ffetarget_convert_logical2_typeless(res,l) \
@@ -981,6 +996,8 @@ void *ffetarget_memcpy_ (void *dst, void *src, size_t len);
 #define ffetarget_convert_logical2_integer1(res,l) (*(res) = (l), FFEBAD)
 #define ffetarget_convert_logical2_integer2(res,l) (*(res) = (l), FFEBAD)
 #define ffetarget_convert_logical2_integer3(res,l) (*(res) = (l), FFEBAD)
+#define ffetarget_convert_logical3_character1(res,l) \
+  ffetarget_convert_any_character1_ ((char *) (res), sizeof(*(res)), l)
 #define ffetarget_convert_logical3_hollerith(res,l) \
   ffetarget_convert_any_hollerith_ ((char *) (res), sizeof(*(res)), l)
 #define ffetarget_convert_logical3_typeless(res,l) \
@@ -990,6 +1007,8 @@ void *ffetarget_memcpy_ (void *dst, void *src, size_t len);
 #define ffetarget_convert_logical3_integer1(res,l) (*(res) = (l), FFEBAD)
 #define ffetarget_convert_logical3_integer2(res,l) (*(res) = (l), FFEBAD)
 #define ffetarget_convert_logical3_integer3(res,l) (*(res) = (l), FFEBAD)
+#define ffetarget_convert_integer1_character1(res,l) \
+  ffetarget_convert_any_character1_ ((char *) (res), sizeof(*(res)), l)
 #define ffetarget_convert_integer1_hollerith(res,l) \
   ffetarget_convert_any_hollerith_ ((char *) (res), sizeof(*(res)), l)
 #define ffetarget_convert_integer1_typeless(res,l) \
@@ -1030,6 +1049,8 @@ void *ffetarget_memcpy_ (void *dst, void *src, size_t len);
 #define ffetarget_convert_integer1_complex1(res,l) (*(res) = (l).real, FFEBAD)
 #define ffetarget_convert_integer1_complex2(res,l) (*(res) = (l).real, FFEBAD)
 #endif
+#define ffetarget_convert_real1_character1(res,l) \
+  ffetarget_convert_any_character1_ ((char *) (res), sizeof(*(res)), l)
 #define ffetarget_convert_real1_hollerith(res,l) \
   ffetarget_convert_any_hollerith_ ((char *) (res), sizeof(*(res)), l)
 #define ffetarget_convert_real1_typeless(res,l) \
@@ -1056,6 +1077,8 @@ void *ffetarget_memcpy_ (void *dst, void *src, size_t len);
 #else
 #define ffetarget_convert_real1_real2(res,l) (*(res) = (l), FFEBAD)
 #endif
+#define ffetarget_convert_real2_character1(res,l) \
+  ffetarget_convert_any_character1_ ((char *) (res), sizeof(*(res)), l)
 #define ffetarget_convert_real2_hollerith(res,l) \
   ffetarget_convert_any_hollerith_ ((char *) (res), sizeof(*(res)), l)
 #define ffetarget_convert_real2_typeless(res,l) \
@@ -1413,7 +1436,7 @@ void *ffetarget_memcpy_ (void *dst, void *src, size_t len);
 #define ffetarget_offset_add(res,l,r) (*(res) = (l) + (r), TRUE)	/* Overflow? */
 #define ffetarget_offset_charsize(res,l,u) (*(res) = (l) * (u), TRUE)	/* Ov? */
 #define ffetarget_offset_multiply(res,l,r) (*(res) = (l) * (r), TRUE)	/* Ov? */
-#define ffetarget_offset_overflow(text)	/* No overflow message issued yet. */
+#define ffetarget_offset_overflow(text) ((void) 0)	/* ~~no message? */
 #define ffetarget_or_integer1(res,l,r) (*(res) = (l) | (r), FFEBAD)
 #define ffetarget_or_integer2(res,l,r) (*(res) = (l) | (r), FFEBAD)
 #define ffetarget_or_integer3(res,l,r) (*(res) = (l) | (r), FFEBAD)
