@@ -52,25 +52,32 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 #include "symbol.h"
 #include "target.h"
 #include "where.h"
+#if FFECOM_targetCURRENT == FFECOM_targetGCC
+#include "flags.h"
+#endif
 
 /* Externals defined here. */
 
 int flag_traditional;		/* Shouldn't need this (C front end only)! */
 bool ffe_is_90_ = FFETARGET_defaultIS_90;
 bool ffe_is_automatic_ = FFETARGET_defaultIS_AUTOMATIC;
+bool ffe_is_backslash_ = FFETARGET_defaultIS_BACKSLASH;
 bool ffe_is_dollar_ok_ = FFETARGET_defaultIS_DOLLAR_OK;
 bool ffe_is_f2c_ = FFETARGET_defaultIS_F2C;
 bool ffe_is_f2c_library_ = FFETARGET_defaultIS_F2C_LIBRARY;
 bool ffe_is_ffedebug_ = FALSE;
 bool ffe_is_free_form_ = FFETARGET_defaultIS_FREE_FORM;
+bool ffe_is_init_local_zero_ = FFETARGET_defaultIS_INIT_LOCAL_ZERO;
 bool ffe_is_mainprog_;		/* TRUE if current prog unit known to be
 				   main. */
 bool ffe_is_pedantic_ = FFETARGET_defaultIS_PEDANTIC;
 bool ffe_is_saveall_;		/* TRUE if mainprog or SAVE (no args) seen. */
 bool ffe_is_ugly_ = FFETARGET_defaultIS_UGLY;
 bool ffe_is_ugly_args_ = FFETARGET_defaultIS_UGLY_ARGS;
+bool ffe_is_ugly_init_ = FFETARGET_defaultIS_UGLY_INIT;
 bool ffe_is_version_ = FALSE;
 bool ffe_is_vxt_not_90_ = FFETARGET_defaultIS_VXT_NOT_90;
+bool ffe_is_warn_implicit_ = FALSE;
 ffeCase ffe_case_intrin_ = FFETARGET_defaultCASE_INTRIN;
 ffeCase ffe_case_match_ = FFETARGET_defaultCASE_MATCH;
 ffeCase ffe_case_source_ = FFETARGET_defaultCASE_SOURCE;
@@ -120,7 +127,9 @@ bool ffe_in_4 = FALSE;
 int
 ffe_decode_option (char *opt)
 {
-  if ((opt[0] == '-') && (opt[1] == 'f'))
+  if (opt[0] != '-')
+    return 0;
+  if (opt[1] == 'f')
     {
       if (strcmp (&opt[2], "version") == 0)
 	ffe_set_is_version (TRUE);
@@ -161,17 +170,37 @@ ffe_decode_option (char *opt)
       else if (strcmp (&opt[2], "f90-not-vxt") == 0)
 	ffe_set_is_vxt_not_90 (FALSE);
       else if (strcmp (&opt[2], "ugly") == 0)
-	ffe_set_is_ugly (TRUE);
+	{
+	  ffe_set_is_ugly (TRUE);
+	  ffe_set_is_ugly_args (TRUE);
+	  ffe_set_is_ugly_init (TRUE);
+	}
       else if (strcmp (&opt[2], "no-ugly") == 0)
-	ffe_set_is_ugly (FALSE);
+	{
+	  ffe_set_is_ugly (FALSE);
+	  ffe_set_is_ugly_args (FALSE);
+	  ffe_set_is_ugly_init (FALSE);
+	}
       else if (strcmp (&opt[2], "ugly-args") == 0)
 	ffe_set_is_ugly_args (TRUE);
       else if (strcmp (&opt[2], "no-ugly-args") == 0)
 	ffe_set_is_ugly_args (FALSE);
+      else if (strcmp (&opt[2], "ugly-init") == 0)
+	ffe_set_is_ugly_init (TRUE);
+      else if (strcmp (&opt[2], "no-ugly-init") == 0)
+	ffe_set_is_ugly_init (FALSE);
       else if (strcmp (&opt[2], "debug") == 0)
 	ffe_set_is_ffedebug (TRUE);
       else if (strcmp (&opt[2], "no-debug") == 0)
 	ffe_set_is_ffedebug (FALSE);
+      else if (strcmp (&opt[2], "init-local-zero") == 0)
+	ffe_set_is_init_local_zero (TRUE);
+      else if (strcmp (&opt[2], "no-init-local-zero") == 0)
+	ffe_set_is_init_local_zero (FALSE);
+      else if (strcmp (&opt[2], "backslash") == 0)
+	ffe_set_is_backslash (TRUE);
+      else if (strcmp (&opt[2], "no-backslash") == 0)
+	ffe_set_is_backslash (FALSE);
       else if (strcmp (&opt[2], "intrin-case-initcap") == 0)
 	ffe_set_case_intrin (FFE_caseINITCAP);
       else if (strcmp (&opt[2], "intrin-case-upper") == 0)
@@ -287,6 +316,43 @@ ffe_decode_option (char *opt)
       else
 	return 0;
     }
+  else if (opt[1] == 'W')
+    {
+      if (!strcmp (&opt[2], "comment"))
+	; /* cpp handles this one.  */
+      else if (!strcmp (&opt[2], "no-comment"))
+	; /* cpp handles this one.  */
+      else if (!strcmp (&opt[2], "comments"))
+	; /* cpp handles this one.  */
+      else if (!strcmp (&opt[2], "no-comments"))
+	; /* cpp handles this one.  */
+      else if (!strcmp (&opt[2], "trigraphs"))
+	; /* cpp handles this one.  */
+      else if (!strcmp (&opt[2], "no-trigraphs"))
+	; /* cpp handles this one.  */
+      else if (!strcmp (&opt[2], "import"))
+	; /* cpp handles this one.  */
+      else if (!strcmp (&opt[2], "no-import"))
+	; /* cpp handles this one.  */
+      else if (!strcmp (&opt[2], "implicit"))
+	ffe_set_is_warn_implicit(TRUE);
+      else if (!strcmp (&opt[2], "no-implicit"))
+	ffe_set_is_warn_implicit(FALSE);
+      else if (!strcmp (&opt[2], "all"))
+	{
+	  extra_warnings = 1;
+	  /* We save the value of warn_uninitialized, since if they put
+	     -Wuninitialized on the command line, we need to generate a
+	     warning about not using it without also specifying -O.  */
+	  if (warn_uninitialized != 1)
+	    warn_uninitialized = 2;
+	  warn_unused = 1;
+	}
+      else
+	return 0;
+    }
+  else if (opt[1] == 'I')
+    return ffecom_decode_include_option (&opt[2]);
   else
     return 0;
 
