@@ -79,18 +79,22 @@ typedef void *ffeUnionLongPtr;	/* unused type to cover union of long and
 
 extern bool ffe_is_90_;
 extern bool ffe_is_automatic_;
+extern bool ffe_is_backslash_;
 extern bool ffe_is_dollar_ok_;
 extern bool ffe_is_f2c_;
 extern bool ffe_is_f2c_library_;
 extern bool ffe_is_ffedebug_;
 extern bool ffe_is_free_form_;
+extern bool ffe_is_init_local_zero_;
 extern bool ffe_is_mainprog_;
 extern bool ffe_is_pedantic_;
 extern bool ffe_is_saveall_;
 extern bool ffe_is_ugly_;
 extern bool ffe_is_ugly_args_;
+extern bool ffe_is_ugly_init_;
 extern bool ffe_is_version_;
 extern bool ffe_is_vxt_not_90_;
+extern bool ffe_is_warn_implicit_;
 extern ffeCase ffe_case_intrin_;
 extern ffeCase ffe_case_match_;
 extern ffeCase ffe_case_source_;
@@ -142,19 +146,23 @@ void ffe_terminate_4 (void);
 #define ffe_intrinsic_state_vxt() ffe_intrinsic_state_vxt_
 #define ffe_is_90() ffe_is_90_
 #define ffe_is_automatic() ffe_is_automatic_
+#define ffe_is_backslash() ffe_is_backslash_
 #define ffe_is_dollar_ok() ffe_is_dollar_ok_
 #define ffe_is_f2c() ffe_is_f2c_
 #define ffe_is_f2c_library() ffe_is_f2c_library_
 #define ffe_is_ffedebug() ffe_is_ffedebug_
 #define ffe_is_free_form() ffe_is_free_form_
+#define ffe_is_init_local_zero() ffe_is_init_local_zero_
 #define ffe_is_mainprog() ffe_is_mainprog_
 #define ffe_is_pedantic() ffe_is_pedantic_
 #define ffe_is_pedantic_not_90() (ffe_is_pedantic_ && !ffe_is_90_)
 #define ffe_is_saveall() ffe_is_saveall_
-#define ffe_is_version() ffe_is_version_
-#define ffe_is_vxt_not_90() ffe_is_vxt_not_90_
 #define ffe_is_ugly() ffe_is_ugly_
 #define ffe_is_ugly_args() ffe_is_ugly_args_
+#define ffe_is_ugly_init() ffe_is_ugly_init_
+#define ffe_is_version() ffe_is_version_
+#define ffe_is_vxt_not_90() ffe_is_vxt_not_90_
+#define ffe_is_warn_implicit() ffe_is_warn_implicit_
 #define ffe_pool_file() (ffe_file_pool_)
 #define ffe_pool_any_unit() (ffe_any_unit_pool_)
 #define ffe_pool_program_unit() (ffe_program_unit_pool_)
@@ -169,18 +177,22 @@ void ffe_terminate_4 (void);
 #define ffe_set_intrinsic_state_vxt(s) (ffe_intrinsic_state_vxt_ = (s))
 #define ffe_set_is_90(f) (ffe_is_90_ = (f))
 #define ffe_set_is_automatic(f) (ffe_is_automatic_ = (f))
+#define ffe_set_is_backslash(f) (ffe_is_backslash_ = (f))
 #define ffe_set_is_dollar_ok(f) (ffe_is_dollar_ok_ = (f))
 #define ffe_set_is_f2c(f) (ffe_is_f2c_ = (f))
 #define ffe_set_is_f2c_library(f) (ffe_is_f2c_library_ = (f))
 #define ffe_set_is_ffedebug(f) (ffe_is_ffedebug_ = (f))
 #define ffe_set_is_free_form(f) (ffe_is_free_form_ = (f))
+#define ffe_set_is_init_local_zero(f) (ffe_is_init_local_zero_ = (f))
 #define ffe_set_is_mainprog(f) (ffe_is_mainprog_ = (f))
 #define ffe_set_is_pedantic(f) (ffe_is_pedantic_ = (f))
 #define ffe_set_is_saveall(f) (ffe_is_saveall_ = (f))
-#define ffe_set_is_version(f) (ffe_is_version_ = (f))
-#define ffe_set_is_vxt_not_90(f) (ffe_is_vxt_not_90_ = (f))
 #define ffe_set_is_ugly(f) (ffe_is_ugly_ = (f))
 #define ffe_set_is_ugly_args(f) (ffe_is_ugly_args_ = (f))
+#define ffe_set_is_ugly_init(f) (ffe_is_ugly_init_ = (f))
+#define ffe_set_is_version(f) (ffe_is_version_ = (f))
+#define ffe_set_is_vxt_not_90(f) (ffe_is_vxt_not_90_ = (f))
+#define ffe_set_is_warn_implicit(f) (ffe_is_warn_implicit_ = (f))
 #define ffe_state_max(s1,s2) ((s1) > (s2) ? (s1) : (s2))
 
 /* End of #include file. */
