@@ -118,7 +118,7 @@ ffewhere_file_begin (ffewhereFile old_wf, ffewhereFile new_wf)
     assert (ffewhere_root_ll_.last->wf == old_wf);
 
   ffewhere_parent_ll_ = ffewhere_root_ll_.last;
-  ffewhere_file_set (new_wf, TRUE, 1);
+  ffewhere_file_set (new_wf, TRUE, 0);
 }
 
 /* Stop reading from a new file, back to an old one.  */
@@ -293,12 +293,13 @@ ffewhere_line_kill (ffewhereLine wl)
 ffewhereLine
 ffewhere_line_new (ffewhereLineNumber ln)
 {
-  ffewhereLine wl;
+  ffewhereLine wl = ffewhere_root_line_.last;
 
   /* If this is the lexer's current line, see if it is already at the end of
      the list, and if not, make it and return it. */
 
-  if (((wl = ffewhere_root_line_.last)->line_num != ln)
+  if (((ln == 0)		/* Presumably asking for EOF pointer. */
+       || (wl->line_num != ln))
       && (ffelex_line_number () == ln))
     {
 #if 0
@@ -322,12 +323,12 @@ ffewhere_line_new (ffewhereLineNumber ln)
 
   /* See if line is on list already. */
 
-  while ((wl != NULL) && (wl->line_num > ln))
+  while (wl->line_num > ln)
     wl = wl->previous;
 
   /* If line is there, increment its use count and return. */
 
-  if ((wl != NULL) && (wl->line_num == ln))
+  if (wl->line_num == ln)
     {
 #if 0
       fprintf (stdout, "; ffewhere_line_new %" ffewhereLineNumber_f "u, uses=%"
